/*
    UPN-Calc - An arbitrary precision RPN calculator for J2ME/MIDP1
    Copyright (C) 2005 Kai G. Schwebke, upn.schwebke.com

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/



/*
 * RPN Calculator Midlet - startup, load and restore of settings and stack
 */

import javax.microedition.midlet.*;
import javax.microedition.lcdui.*;
import javax.microedition.rms.*;
import java.io.*;


public class UPNMIDlet
   extends MIDlet
{
   private UPNCanvas canvas;


   public UPNMIDlet()
   {
      canvas = new UPNCanvas(this, Display.getDisplay(this));
   }


   public void startApp()
   {
      Display.getDisplay(this).setCurrent(canvas);
      canvas.start();

      RecordStore r;
      try {
         r = RecordStore.openRecordStore("settings", true);
	 if (r.getNumRecords() > 0)
	 {
	    ByteArrayInputStream bais =
	      new ByteArrayInputStream(r.getRecord(1));
	    DataInputStream inputStream = new DataInputStream(bais);
	    try {
	       canvas.setAngleMeasure(inputStream.readUTF());
	       canvas.setFontSize(inputStream.readUTF());
	       canvas.setStackPrec(inputStream.readInt());
	       canvas.setDefaultPrec(inputStream.readInt());

	       canvas.getStack().clear();
	       int n = inputStream.readInt();
	       int i;
	       for (i = 0; i < n; ++i)
	       {
		  canvas.getStack().push(new BCDFloat(inputStream.readUTF()));
	       }
	       canvas.repaint();
	    } catch (EOFException eofe) {
	       System.out.println(eofe);
	       eofe.printStackTrace();
	    }
	 }
	 r.closeRecordStore();
      } catch (RecordStoreException rse) {
	 System.out.println(rse);
	 rse.printStackTrace();
      } catch (IOException ioe) {
	 System.out.println(ioe);
	 ioe.printStackTrace();
      }
   }


   public void pauseApp()
   {
      canvas.stop();
   }


   public void destroyApp(boolean unconditional)
   {
      canvas.stop();
   }


   void exitRequested()
   {
      RecordStore r;
      try {
         r = RecordStore.openRecordStore("settings", true);
	 ByteArrayOutputStream baos = new ByteArrayOutputStream();
	 DataOutputStream outputStream = new DataOutputStream(baos);
	 outputStream.writeUTF(canvas.getAngleMeasure());
	 outputStream.writeUTF(canvas.getFontSize());
	 outputStream.writeInt(canvas.getStackPrec());
	 outputStream.writeInt(canvas.getDefaultPrec());

	 int n = canvas.getStack().size();
	 outputStream.writeInt(n);
	 int i;
	 for (i = 0; i < n; ++i)
	 {
	    outputStream.writeUTF(canvas.getStack().at(i).toString());
	 }
	 byte[] b = baos.toByteArray();
	 if (r.getNumRecords() > 0)
	 {
	    r.setRecord(1, b, 0, b.length);
	 } else {
	    r.addRecord(b, 0, b.length);
	 }
	 r.closeRecordStore();
      } catch (RecordStoreException rse) {
	 System.out.println(rse);
	 rse.printStackTrace();
      } catch (IOException ioe) {
	 System.out.println(ioe);
	 ioe.printStackTrace();
      }
      
      destroyApp(false);
      notifyDestroyed();
   }
}

